#include <avr/interrupt.h>
#include <avr/io.h>
#include <string.h>
#include "config.h"
#include "spi.h"

#define DD_SS 	PB4
#define DD_MOSI	PB5
#define DD_SCK 	PB7

#define SYNC 	0xaa

typedef void *(*SpiStateFunc)(void);

typedef union  {
	uint16_t s;
	struct {
		uint8_t b0;
		uint8_t b1;
	} b;
} SB;

void *sendSync(void);
void *sendAddr(void);
void *sendHiData(void);
void *sendLoData(void);

uint16_t reg[NREG];
uint8_t regUpdate[NREG];
int8_t oAddr;
int8_t iAddr;
SB oData;
SB iData;
SpiStateFunc spiStateFunc;
uint8_t updatePos;
int8_t updateAddr;


void initSpi(void) {
	DDRB |= (1<<DD_SS)| (1<<DD_SCK) | (1<<DD_MOSI); 
	SPCR = (1<<SPIE)|(1<<SPE)|(1<<MSTR)|(1<<SPR0);  // irq, enable, master, clock/16

	oAddr = 0;
	iAddr = 0;
	updateAddr = 0;
	memset(reg, 0, sizeof(reg));
	memset(regUpdate, 0, sizeof(regUpdate));
	updatePos = 0;
	spiStateFunc = sendSync(); 
}

void spiUpdate() {
	uint8_t i;

	if (iAddr != REG_AUTO)
		return;

	for (i=0;i<NREG;i++) {
		if (regUpdate[updatePos]!=0) {
			regUpdate[updatePos] = 0;
			updateAddr = updatePos;
			updatePos++;
			updatePos &= 0x7f;
			break;
		}
		updatePos++;
		updatePos &= 0x7f;
	}
}

void *sendSync() {

	iData.b.b1 = SPDR;

	SPDR = SYNC;
	if (iAddr & 0x80)
		reg[iAddr & 0x7f] = iData.s;
	else 
		oAddr = iAddr;

	if (iAddr == REG_AUTO)
		oAddr = updateAddr;
	return sendAddr;
};

void *sendAddr() {
	SPDR = oAddr;
	oData.s = reg[oAddr];
	return sendHiData;
}

void *sendHiData() {
	iAddr = SPDR;
	SPDR = oData.b.b1;
	return sendLoData;
}

void *sendLoData() {
	iData.b.b0 = SPDR;
	SPDR = oData.b.b0;
	return sendSync;
}

ISR(SPI_STC_vect) {
	spiStateFunc = (SpiStateFunc)(*spiStateFunc)();
}
