#include <avr/interrupt.h>
#include <avr/io.h>
#include "config.h"
#include "timer0.h"
#include "types.h"
#include "adc.h"
#include "encoder.h"

static int16_t last;
static int8_t last1;
static int8_t last2;

void timer0_init (void) {
	int16_t new;

	TCCR0 = 1<<WGM01^1<<CS01^1<<CS00;     // CTC, XTAL / 64
	OCR0 = (uint8_t)(SYSCLK / 64.0 * 100e-6 - 0.5);   // 0.1ms
	TIMSK |= 1<<OCIE0;

	new = 0;
	if( PHASE_A )
		new = 3;
	if( PHASE_B )
		new ^= 1;                   // convert gray to binary
	last = new;                   // power on state
	
	new = 0;
	if( PHASE_A1 )
		new = 3;
	if( PHASE_B1 )
		new ^= 1;                   // convert gray to binary
	last1 = new;                   // power on state

	new = 0;
	if( PHASE_A2 )
		new = 3;
	if( PHASE_B2 )
		new ^= 1;                   // convert gray to binary
	last2 = new;                   // power on state
    return;
}


ISR( TIMER0_COMP_vect )
{
	int16_t new, diff;
	int8_t new1, diff1;
	int8_t new2, diff2;

	new = 0;
	if( PHASE_A )
		new = 3;
	if( PHASE_B )
		new ^= 1;                   // convert gray to binary
	diff = last - new;                // difference last - new
	if( diff & 1 ){               // bit 0 = value (1)
		last = new;                 // store new as next last
		enc_delta += (diff & 2) - 1;        // bit 1 = direction (+/-)
	}

	new1 = 0;
	if( PHASE_A1 )
		new1 = 3;
	if( PHASE_B1 )
		new1 ^= 1;
	diff1 = last1 - new1;
	if( diff1 & 1 ) {
		last1 = new1;
		enc_delta1 += (diff1 & 2) - 1;
	}

	new2 = 0;
	if( PHASE_A2 )
		new2 = 3;
	if( PHASE_B2 )
		new2 ^= 1;
	diff2 = last2 - new2;
	if( diff2 & 1 ) {
		last2 = new2;
		enc_delta2 += (diff2 & 2) - 1;
	}
}



